#include<iostream>
#include<string.h>
#include<fstream>
#include<algorithm>
#include<math.h>
#include<vector>
#include <stdlib.h>
#include <stdio.h>

using namespace std;

#define veryLongInt long long int

veryLongInt getSamplingSize(int dim, int kValue, double epsilonValue, double deltaValue)
{
	
	double sOne = dim - 1;
	
	double returnValueDouble = sOne * 8.0 * kValue*kValue * log( 2.0 / ( pow( (deltaValue*0.5), (1.0 / kValue) ) ) ) / (epsilonValue*epsilonValue);
	
	veryLongInt returnValue = floor(returnValueDouble);
	
	return returnValue;
}

int main(int argc, char * argv[])
{
	int kInput = 1;
	if(argc != 4)
	{
		cout<<"The input format is wrong. Please re-run the program."
		return;
	}
	kInput = atoi(argv[1]);
	
	string filename = argv[2];
	
	int dim, size;
	ifstream inFile.open(filename.c_str());
	inFile >> dim;
	inFile >> size;
	
	double** raw = new double*[size];
	int* utilCount = new int[size];
	bool* isSky = new bool[size];
	bool* isResult = new bool[size];
	for(int i = 0; i < size; i++)
	{
		raw[i] = new double[dim];
		isSky[i] = true;
		isResult[i] = false;
		utilCount[i] = 0;
		for(int j = 0; j < dim; j++)
		{
			inFile >> raw[i][j];
		}
	}
	inFile.close();
	
	int count;
	bool isUnder;
	for(int i = 0; i < size; i++)
	{
		if(!isSky[i]) continue;
		
		for(int j = 0; j < size; j++)
		{
			if(i != j)
			{
				count = 0;
				isUnder = false;
				for(int d = 0; d < dim; d++)
				{
					if(raw[i][d] >= raw[j][d])
					{
						count++;
						if(raw[i][d] > raw[j][d]) isUnder = true;
					}
				}
				
				if(count == dim && isUnder)
				{
					isSky[j] = false;
				}
			}
		}
	}
	
	if(kInput >= size)
	{
		char inputChar;
		cout<< "The value of k is equal or greater than the data size, continue? (y/n)"<<endl;
		cin >> inputChar;
		if(inputChar == 'y')
		{
			cout << "Exit!" << endl;
			return 100;
		}
		kInput = size;
	}
	
	veryLongInt samplingSize = getSamplingSize(dim, kInput, 0.0001 /*epilson*/, 0.1 /*delta*/);
	cout << "Value of k: " << kInput <<endl;
	cout << "Sampling Size: " << samplingSize << endl;
	
	
	
	//the samples (i.e., sampled utility functions)
	string filename_samples = argv[3];
	ifstream inUtilFile.open(filename_samples.c_str);
	double** utilFuncArray = new double*[size];
	double* utilDimMax = new double[dim];
	double* utilDimMin = new double[dim];
	int* utilFuncMaxId = new int[size];
	for(int i = 0; i < samplingSize; i++)
	{
		utilFuncArray[i] = new double[dim];
		utilFuncMaxId[i] = -1;
		for(int j = 0; j < dim; j++)
		{
			inUtilFile >> utilFuncArray[i][j];
		}
	}
	inUtilFile.close();
	
	cout << "-----------------" << endl;
	for(int i = 0; i < samplingSize; i++)
	{
		for(int d = 0; d < dim; d++)
		{
			cout << utilFuncArray[i][d] << " ";
		}
		cout << endl;
	}
	cout << "-----------------" << endl;
	
	for(int d = 0; d < dim; d++)
	{
		for(int i = 0; i < samplingSize; i++)
		{
			if(i==0)
			{	
				utilDimMax[d] = utilFuncArray[i][d];
				utilDimMin[d] = utilFuncArray[i][d];
			}
			else
			{
				if(utilDimMax[d] < utilFuncArray[i][d]) utilDimMax[d] = utilFuncArray[i][d];
				if(utilDimMin[d] > utilFuncArray[i][d]) utilDimMin[d] = utilFuncArray[i][d];
			}
		}
	}
		
	//double utilSqSumRt;
	for(int i = 0; i < size; i++)
	{
		//utilSqSumRt = 0.0;
		for(int j = 0; j < dim; j++)
		{	
			utilFuncArray[i][j] = utilFuncArray[i][j] - utilDimMin[j];
			//utilSqSumRt += utilFuncArray[i][j] * utilFuncArray[i][j];
		}
		//normalize
		//utilSqSumRt = sqrt(utilSqSumRt);
	}
	
	cout << "***************" << endl;
	for(int i = 0; i < samplingSize; i++)
	{
		for(int d = 0; d < dim; d++)
		{
			cout << utilFuncArray[i][d] << " ";
		}
		cout << endl;
	}
	cout << "***************" << endl;
	
	double tempDot;
	double tempDotMax;
	for(int k = 0; k < samplingSize; k++)
	{
		for(int i = 0; i < size /*raw size*/; i++)
		{	
			tempDot = 0.0;
			for(int j = 0; j < dim; j++)
			{
				tempDot += utilFuncArray[k][j] * raw[i][j];
			}
			
			if(i == 0)
			{
				utilFuncMaxId[k] = i;
				tempDotMax = tempDot;
			}
			else if(tempDot > tempDotMax)	
			{
				utilFuncMaxId[k] = i;
				tempDotMax = tempDot;
			}
		}
		
		utilCount[utilFuncMaxId[k]]++;
	}
	
	int tempMaxCount;
	int tempID;
	int utilNum = samplingSize;
	for(int k = 0; k < kInput; k++)
	{	
		tempMaxCount = -1;
		for(int i = 0; i < size; i++)
		{	
			if(!isResult[i])
			{
				if(utilCount[i] > tempMaxCount)
				{
					tempMaxCount = utilCount[i];
					tempID = i;
				}
			}
		}
		isResult[tempID] = true;
	}
	
	int outputUtilNum = 0;
	cout<<"Output Point: ";
	for(int i = 0; i < size; i++)
	{	
		if(isResult[i])
		{
			outputUtilNum += utilCount[i];
			cout << i << " ";
			for(int j = 0; j < dim; j++)
			{
				cout << raw[i][j] << " ";
			}
			cout << endl;
		}
	}
	
	cout << "Hit Ratio: " <<  ((double) outputUtilNum) / utilNum << endl;
	return 0;
}
